@@ -13,6 +13,14 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef __GLIBC__
+#include <locale.h>
+#include <iconv.h>
+#include <langinfo.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <unistd.h>
+#endif
 #include "gnome-defs.h"
 #include "gnome-util.h"
 #include "gnome-url.h"
@@ -20,6 +28,11 @@
 #include "gnome-dentry.h"
 #include "gnome-exec.h"
 
+#ifdef __GLIBC__
+static char *utf8tolocalechar (const char *value);
+static gboolean file_looks_utf8 (const char *filename);
+#endif
+
 /* g_free already checks if x is NULL */
 #define free_if_empty(x) g_free (x)
 
@@ -134,17 +147,25 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 	GnomeDesktopEntry *newitem;
 	char *prefix;
 	char *name, *type;
+#ifdef __GLIBC__
+	char *comment;
+#endif
 	char *try_file;
 	char **exec_vector;
 	int exec_length;
 	char *icon_base;
 	char *p = NULL;
 	gboolean is_kde = FALSE;
-	
+ 	gboolean is_dot_directory = FALSE;
+#ifdef __GLIBC__
+	gboolean is_utf8;
+#endif
 	GList *i18n_list = NULL;
 	
 	g_assert (file != NULL);
-	
+#ifdef __GLIBC__
+	is_utf8 = file_looks_utf8 (file);
+#endif
 	prefix = g_strconcat ("=", file, "=/Desktop Entry/", NULL);
 	gnome_config_push_prefix (prefix);
 	g_free (prefix);
@@ -172,7 +193,11 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 	 * entries, we will later need to make this code smarter.
 	 */
 
-	type      = gnome_config_get_string (is_kde ? "Type=Directory" : "Type");
+	if (strlen (file) >= strlen (".directory") &&
+	    strcmp (file + strlen (file) - strlen (".directory"), ".directory") == 0)
+		is_dot_directory = TRUE;
+	
+	type      = gnome_config_get_string (is_dot_directory ? "Type=Directory" : "Type");
 	gnome_config_get_vector ("Exec", &exec_length, &exec_vector);
 	try_file  = gnome_config_get_string ("TryExec");
 
@@ -193,8 +218,28 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 	
 	newitem = g_new0 (GnomeDesktopEntry, 1);
 
-	newitem->name          = name;
-	newitem->comment       = gnome_config_get_translated_string ("Comment");
+#ifdef __GLIBC__	
+	comment = gnome_config_get_translated_string ("Comment");
+	if (is_utf8) {
+		if (name) {
+			newitem->name = utf8tolocalechar (name);
+			g_free (name);
+		} else
+			newitem->name = NULL;
+
+		if (comment) {
+			newitem->comment = utf8tolocalechar (comment);
+			g_free (comment);
+		} else
+			newitem->comment = NULL;
+	} else {
+		newitem->name = name;
+		newitem->comment = comment;
+	}
+#else
+	newitem->name = name;
+	newitem->comment = gnome_config_get_translated_string ("Comment");
+#endif	
 	newitem->exec_length   = exec_length;
 	newitem->exec          = exec_vector;
 	newitem->tryexec       = try_file;
@@ -219,8 +264,23 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 					g_free (iconname);
 					newitem->icon = NULL;
 				}
-			} else
+			} else {
 				newitem->icon = gnome_pixmap_file (icon_base);
+				/* If it wasn't found, look in kde */
+				if (!newitem->icon) {
+					gchar *iconname = g_concat_dir_and_file (KDE_ICONDIR "/hicolor/32x32/apps", icon_base);
+					if (g_file_exists (iconname))
+						newitem->icon = iconname;
+					else {	/* Maybe it didn't have an extension? */
+						gchar *icon2 = g_strconcat (iconname, ".png", NULL);
+						if (g_file_exists (icon2))
+							newitem->icon = icon2;
+						else
+							g_free (icon2);
+						g_free (iconname);
+					}
+				}
+			}
 			g_free (icon_base);
 		} else
 			newitem->icon = icon_base;
@@ -536,9 +596,7 @@ gnome_desktop_entry_launch_with_args (GnomeDesktopEntry *item, int the_argc, cha
 
 	g_assert (item != NULL);
 
-	if (!item->terminal && the_argc == 0 && !item->is_kde)
-	    exec_str = g_strjoinv (" ", (char **)(item->exec));
-	else {
+	{
 		if (item->terminal) {
 			gnome_config_get_vector ("/Gnome/Applications/Terminal",
 						 &term_argc, &term_argv);
@@ -570,13 +628,10 @@ gnome_desktop_entry_launch_with_args (GnomeDesktopEntry *item, int the_argc, cha
 
 		/* ... arguments from the desktop file */
 		for (i = 0; i < item->exec_length; ++i) {
-			if (item->is_kde) {
-				sub_arg = gnome_desktop_entry_sub_kde_arg (item, item->exec[i]);
-				if (sub_arg) {
-					args_to_free = g_slist_prepend (args_to_free, sub_arg);
-					argv[term_argc + i] = sub_arg;
-				} else
-					argv[term_argc + i] = item->exec[i];
+			sub_arg = gnome_desktop_entry_sub_kde_arg (item, item->exec[i]);
+			if (sub_arg) {
+				args_to_free = g_slist_prepend (args_to_free, sub_arg);
+				argv[term_argc + i] = sub_arg;
 			} else
 				argv[term_argc + i] = item->exec[i];
 		}
@@ -809,3 +864,240 @@ gnome_desktop_entry_free_i18n_list(GList *list)
 	}
 	if(list) g_list_free(list);
 }
+
+#ifdef __GLIBC__
+static gboolean
+get_contents_regfile (const gchar *filename,
+                      struct stat *stat_buf,
+                      gint         fd,
+                      gchar      **contents,
+                      gsize       *length)
+{
+  gchar *buf;
+  size_t bytes_read;
+  size_t size;
+      
+  size = stat_buf->st_size;
+
+  buf = g_new (gchar, size + 1);
+      
+  bytes_read = 0;
+  while (bytes_read < size)
+    {
+      gssize rc;
+          
+      rc = read (fd, buf + bytes_read, size - bytes_read);
+
+      if (rc < 0)
+        {
+          if (errno != EINTR) 
+            {
+              close (fd);
+
+              g_free (buf);
+                  
+              return FALSE;
+            }
+        }
+      else if (rc == 0)
+        break;
+      else
+        bytes_read += rc;
+    }
+      
+  buf[bytes_read] = '\0';
+
+  if (length)
+    *length = bytes_read;
+  
+  *contents = buf;
+
+  close (fd);
+
+  return TRUE;
+}
+
+/* Cut and paste from Glib-2.0
+ */
+static gboolean
+g_file_get_contents (const gchar *filename,
+                     gchar      **contents,
+                     gsize       *length)
+{  
+  struct stat stat_buf;
+  gint fd;
+  
+  g_return_val_if_fail (filename != NULL, FALSE);
+  g_return_val_if_fail (contents != NULL, FALSE);
+
+  *contents = NULL;
+  if (length)
+    *length = 0;
+
+  fd = open (filename, O_RDONLY);
+
+  if (fd < 0)
+    return FALSE;
+
+  /* I don't think this will ever fail, aside from ENOMEM, but. */
+  if (fstat (fd, &stat_buf) < 0)
+    {
+      close (fd);
+      
+      return FALSE;
+    }
+
+  if (stat_buf.st_size > 0 && S_ISREG (stat_buf.st_mode))
+    {
+      return get_contents_regfile (filename,
+                                   &stat_buf,
+                                   fd,
+                                   contents,
+                                   length);
+    }
+  else
+    {
+      /* Glib-2.0 includes fallback to stdio in this case, but this shouldn't
+       * happen for desktop files
+       */
+
+      return FALSE;
+    }
+}
+
+#define F 0   /* character never appears in text */
+#define T 1   /* character appears in plain ASCII text */
+#define I 2   /* character appears in ISO-8859 text */
+#define X 3   /* character appears in non-ISO extended ASCII (Mac, IBM PC) */
+
+static char text_chars[256] = {
+	/*                  BEL BS HT LF    FF CR    */
+	F, F, F, F, F, F, F, T, T, T, T, F, T, T, F, F,  /* 0x0X */
+        /*                              ESC          */
+	F, F, F, F, F, F, F, F, F, F, F, T, F, F, F, F,  /* 0x1X */
+	T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x2X */
+	T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x3X */
+	T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x4X */
+	T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x5X */
+	T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x6X */
+	T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, F,  /* 0x7X */
+	/*            NEL                            */
+	X, X, X, X, X, T, X, X, X, X, X, X, X, X, X, X,  /* 0x8X */
+	X, X, X, X, X, X, X, X, X, X, X, X, X, X, X, X,  /* 0x9X */
+	I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xaX */
+	I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xbX */
+	I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xcX */
+	I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xdX */
+	I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xeX */
+	I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I   /* 0xfX */
+};
+
+static gboolean
+file_looks_utf8 (const char *filename)
+{
+	int i, n;
+	unsigned long c;
+	char *buf;
+	size_t nbytes;
+	gboolean gotone = FALSE;
+	gboolean result = TRUE;
+
+	if (!g_file_get_contents (filename, &buf, &nbytes))
+		return FALSE;
+
+	for (i = 0; result && i < nbytes; i++) {
+		if ((buf[i] & 0x80) == 0) {	   /* 0xxxxxxx is plain ASCII */
+			/*
+			 * Even if the whole file is valid UTF-8 sequences,
+			 * still reject it if it uses weird control characters.
+			 */
+
+			if (text_chars[((guchar *)buf)[i]] != T)
+				result = FALSE;
+		} else if ((buf[i] & 0x40) == 0) { /* 10xxxxxx never 1st byte */
+			result = FALSE;
+		} else {			   /* 11xxxxxx begins UTF-8 */
+			int following;
+
+			if ((buf[i] & 0x20) == 0) {		/* 110xxxxx */
+				c = buf[i] & 0x1f;
+				following = 1;
+			} else if ((buf[i] & 0x10) == 0) {	/* 1110xxxx */
+				c = buf[i] & 0x0f;
+				following = 2;
+			} else if ((buf[i] & 0x08) == 0) {	/* 11110xxx */
+				c = buf[i] & 0x07;
+				following = 3;
+			} else if ((buf[i] & 0x04) == 0) {	/* 111110xx */
+				c = buf[i] & 0x03;
+				following = 4;
+			} else if ((buf[i] & 0x02) == 0) {	/* 1111110x */
+				c = buf[i] & 0x01;
+				following = 5;
+			} else {
+				result = FALSE;
+				goto done;
+			}
+
+			for (n = 0; n < following; n++) {
+				i++;
+				if (i >= nbytes) {
+					result = FALSE;
+					goto done;
+				}
+
+				if ((buf[i] & 0x80) == 0 || (buf[i] & 0x40)) {
+					result = FALSE;
+					goto done;
+				}
+
+				c = (c << 6) + (buf[i] & 0x3f);
+			}
+
+			gotone = TRUE;
+		}
+	}
+done:
+	g_free (buf);
+
+	return result && gotone;   /* don't claim it's UTF-8 if it's all 7-bit */
+}
+
+static char *
+utf8tolocalechar (const char *value)
+{
+	static gboolean initialized = FALSE;
+	static iconv_t fd = (iconv_t)-1;
+	
+	if (!initialized) {
+		gchar *locale;
+		
+		setlocale (LC_CTYPE, "");
+		locale = nl_langinfo (CODESET);
+
+		if (g_strcasecmp (locale, "utf8") != 0 && g_strcasecmp (locale, "utf8") != 0) {
+			fd = iconv_open (locale, "UTF-8");
+			if (fd == (iconv_t)-1) {
+				g_warning ("gnome-dentry.c: Cannot open converter from UTF-8 to locale");
+			}
+		}
+		
+		initialized = TRUE;
+	}
+	if (fd != (iconv_t)-1) {
+		gchar *pout, *pin, *str;
+		gint len, ib, ob;
+
+		len = strlen (value);
+
+		ib = len;
+		ob = ib * 3;
+		pout = str = g_new0 (gchar, ob);
+		pin = (char *)value;
+		iconv (fd, NULL, NULL, &pout, &ob);
+		iconv (fd, &pin, &ib, &pout, &ob);
+		return str;
+	} else
+	        return g_strdup (value);
+}
+#endif
