@@ -80,7 +80,7 @@ day_selected (GtkCalendar *calendar, GnomeDateEdit *gde)
 
 	gtk_calendar_get_date (calendar, &year, &month, &day);
 
-	date = g_date_new_dmy (day, month, year);
+	date = g_date_new_dmy (day, month + 1, year);
 	g_date_strftime (buffer, sizeof(buffer), "%x", date);
 	g_date_free (date);
 
@@ -259,10 +259,7 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 		mtm->tm_min  = 0;
 		hit = g_new (hour_info_t, 1);
 
-		if (gde->flags & GNOME_DATE_EDIT_24_HR)
-			strftime (buffer, sizeof (buffer), "%H:00", mtm);
-		else
-			strftime (buffer, sizeof (buffer), "%I:00 %p ", mtm);
+		strftime (buffer, sizeof (buffer), "%X", mtm);
 		hit->hour = g_strdup (buffer);
 		hit->gde  = gde;
 
@@ -283,10 +280,7 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 
 			mtm->tm_min = j;
 			hit = g_new (hour_info_t, 1);
-			if (gde->flags & GNOME_DATE_EDIT_24_HR)
-				strftime (buffer, sizeof (buffer), "%H:%M", mtm);
-			else
-				strftime (buffer, sizeof (buffer), "%I:%M %p", mtm);
+			strftime (buffer, sizeof (buffer), "%X", mtm);
 			hit->hour = g_strdup (buffer);
 			hit->gde  = gde;
 
@@ -403,10 +397,7 @@ gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 	gtk_entry_set_text (GTK_ENTRY (gde->date_entry), buffer);
 
 	/* Set the time */
-	if (gde->flags & GNOME_DATE_EDIT_24_HR)
-		strftime (buffer, sizeof (buffer), "%H:%M", mytm);
-	else
-		strftime (buffer, sizeof (buffer), "%I:%M %p", mytm);
+	strftime (buffer, sizeof (buffer), "%X", mytm);
 	gtk_entry_set_text (GTK_ENTRY (gde->time_entry), buffer);
 }
 
