@@ -129,7 +129,7 @@ int vt_line_mblen(int x, struct vt_line *l)
       if (len <= 0) len = 1;
   }
 
-  return(len);
+  return len;
 }
 
 int vt_query_line_mbchar(int x, struct vt_line *l)
@@ -137,7 +137,7 @@ int vt_query_line_mbchar(int x, struct vt_line *l)
   unsigned char ctmp[MB_CUR_MAX];
   int xx = x, len = 1, i;
 
-  if (x == 0 || x == l->width) return(x);
+  if (x == 0 || x == l->width) return x;
 
   /* arranged selection columns (sx and ex) for multibyte character */
   if (MB_CUR_MAX >= 2) {
@@ -161,7 +161,7 @@ int vt_query_line_mbchar(int x, struct vt_line *l)
 	  }
       }
   }
-  return(xx);
+  return xx;
 }
 #endif
  
